/***********************************************************************

GECCO, a platform for command and control gaming

Copyright (C) 2001 Henrik Brnhielm, Andreas Enblom, Niklas
Hallenfur, Karl Hasselstrm, Henrik Hgerstrm, Oskar Linde, Jing Fu
Zi and Jon slund

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
USA.

***********************************************************************/

package gecco.client;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.net.*;
import java.util.*;
import javax.swing.*;

/**
 *  The board is what is displayed to the user of the game. It paints the map
 *  and the pieces. The board needs to know of a Game object, whose pieces are
 *  to be painted on the board. <p>
 *
 *  The board contains scrollbars used for scrolling when the map does not fit
 *  the screen. It also contain zoom buttons for zooming in or out on the board.
 *  The zoom buttons use some gif images in the icons catalog (zoomin.gif and
 *  zoomout.gif). <p>
 *
 *  All painting is done in an offscreen buffer and is then copied to the
 *  screen. Use method repaintBoard, to repaint the board to reflect changes in
 *  the game.
 *
 * @author     Andreas Enblom
 * @author     Karl Hasselstrm
 * @created    June 27, 2001
 */
public class Board extends JPanel {

    /**
     *  The Game object that keeps track of the pieces.
     */
    private Game game;
    /**
     *  The "board" part of the board (the part where the map is drawn).
     */
    private JComponent board;
    /**
     *  The scrollbars.
     */
    private JScrollBar horizontalScrollbar, verticalScrollbar;
    /**
     *  The adjustment listeners that update things when the user slides the
     *  scroll bars.
     */
    private AdjustmentListener horScrollListener, vertScrollListener;
    /**
     *  The zoom in and zoom out buttons.
     */
    private JButton zoomInButton, zoomOutButton;

    /**
     *  The offscreen buffer everything is drawn in. When we're done drawing,
     *  the contents of this buffer is pasted onto the board JComponent.
     */
    private Image offscreenBuffer;
    /**
     *  The zoom factor. 0 means natural size (100% magnification), 1 means 200%
     *  magnification, -1 means 50% magnification (i.e. half the natural size),
     *  etc.
     */
    private int zoom;
    /**
     *  The map we're supposed to draw.
     */
    private Map map;
    /**
     *  The uppermost, leftmost point in the automaton map that fits in the
     *  window.
     */
    private Position mapUL;
    /**
     *  The point on the board where the upper left corner of the map is.
     */
    private Point boardUL;

    private boolean repaint = false;



    /**
     * @param  game  The Game object whose pieces are supposed to be painted on
     *      this board.
     */
    public Board(Game game) {
        this.game = game;

        // Offscreen buffer. The only thing we know for sure of its
        // size is that it won't need to be bigger than the entire
        // physical screen.
        Dimension bufferSize = Toolkit.getDefaultToolkit().getScreenSize();
        offscreenBuffer = new BufferedImage(bufferSize.width, bufferSize.height,
                BufferedImage.TYPE_3BYTE_BGR);

        // The board JComponent
        board =
            new JComponent() {
                /**
                 *  The minimum size of the map JComponent is 200 by 200 pixels.
                 *
                 * @return    The MinimumSize value
                 */
                public Dimension getMinimumSize() {
                    return new Dimension(200, 200);
                }


                /**
                 *  Moves and resizes the board.
                 *
                 * @param  x       Description of Parameter
                 * @param  y       Description of Parameter
                 * @param  width   Description of Parameter
                 * @param  height  Description of Parameter
                 */
                public void reshape(int x, int y, int width, int height) {
                    if (map != null) {
                        Position center = map.getCenter();
                        super.reshape(x, y, width, height);
                        adjustZoom(center);
                    } else {
                        super.reshape(x, y, width, height);
                    }
                }


                /**
                 *  Draws the map JComponent using the given Graphics.
                 *
                 * @param  g  The Graphics object to be used for drawing.
                 */
                public void paint(Graphics g) {
                    repaintOffscreenBuffer(getSize());
                    g.drawImage(offscreenBuffer, 0, 0, this);
                }
            };

        // scrollbars
        horizontalScrollbar = new JScrollBar(Scrollbar.HORIZONTAL);
        horScrollListener =
            new AdjustmentListener() {
                /**
                 *  Description of the Method
                 *
                 * @param  e  Description of Parameter
                 */
                public void adjustmentValueChanged(AdjustmentEvent e) {
                    mapUL.x = (double) e.getValue();
                    board.repaint();
                }
            };
        verticalScrollbar = new JScrollBar(Scrollbar.VERTICAL);
        vertScrollListener =
            new AdjustmentListener() {
                /**
                 *  Description of the Method
                 *
                 * @param  e  Description of Parameter
                 */
                public void adjustmentValueChanged(AdjustmentEvent e) {
                    mapUL.y = (double) e.getValue();
                    board.repaint();
                }
            };

        // zoom buttons
        zoomInButton =
            new JButton() {
                /**
                 *  Gets the PreferredSize attribute of the Board object
                 *
                 * @return    The PreferredSize value
                 */
                public Dimension getPreferredSize() {
                    int size = this.getMinimumSize().height;
                    return new Dimension(size, size);
                }
            };
        zoomInButton.setIcon(new ImageIcon(
                ClassLoader.getSystemClassLoader().
                getResource("gecco/client/icons/zoomin.gif")));
        zoomInButton.addActionListener(
            new ActionListener() {
                /**
                 *  Description of the Method
                 *
                 * @param  e  Description of Parameter
                 */
                public void actionPerformed(ActionEvent e) {
                    zoomIn();
                    board.repaint();
                }
            });
        zoomOutButton =
            new JButton() {
                /**
                 *  Gets the PreferredSize attribute of the Board object
                 *
                 * @return    The PreferredSize value
                 */
                public Dimension getPreferredSize() {
                    int size = this.getMinimumSize().height;
                    return new Dimension(size, size);
                }
            };
        zoomOutButton.setIcon(new ImageIcon(
                ClassLoader.getSystemClassLoader().
                getResource("gecco/client/icons/zoomout.gif")));
        zoomOutButton.addActionListener(
            new ActionListener() {
                /**
                 *  Description of the Method
                 *
                 * @param  e  Description of Parameter
                 */
                public void actionPerformed(ActionEvent e) {
                    zoomOut();
                    board.repaint();
                }
            });

        // layout
        Box lower = new Box(BoxLayout.X_AXIS);
        zoomInButton.setAlignmentY(Component.TOP_ALIGNMENT);
        zoomOutButton.setAlignmentY(Component.TOP_ALIGNMENT);
        horizontalScrollbar.setAlignmentY(Component.TOP_ALIGNMENT);
        lower.add(zoomInButton);
        lower.add(zoomOutButton);
        lower.add(horizontalScrollbar);
        setLayout(new BorderLayout());
        add(board, BorderLayout.CENTER);
        add(verticalScrollbar, BorderLayout.EAST);
        add(lower, BorderLayout.SOUTH);

        // The zoom buttons and scroll bars should be disbabled until
        // a map has been loaded.
        horizontalScrollbar.setEnabled(false);
        verticalScrollbar.setEnabled(false);
        zoomInButton.setEnabled(false);
        zoomOutButton.setEnabled(false);

	// The repaint thread (ugly hack to speed up the client when
	// running Windows)
	Thread repainter =
	    new Thread("Repainter thread") {
		    public void run() {
			while(true) {
			    try {
				sleep(100);
				if (shouldRepaint()) {
				    clearRepaint();
				    board.repaint();
				}
			    } catch (InterruptedException e) {
				continue;
			    }
			}
		    }
		    
		    private boolean shouldRepaint() {
			synchronized (Board.this) {
			    return repaint;
			}
		    }
		    
		    private void clearRepaint() {
			synchronized (Board.this) {
			    repaint = false;
			}
		    }
		};
	repainter.start();
    }


    /**
     *  Set the cursor for the map area of the board.
     *
     * @param  cursor  The new BoardCursor value
     */
    public void setBoardCursor(Cursor cursor) {
        board.setCursor(cursor);
    }


    /**
     *  Converts a screen point to the Position corresponding to that point.
     *  Things like zoom factor and scrollbar position are taken into account.
     *
     * @param  p  A pixel in the board coordinate system.
     * @return    The corresponding position in the map coordinate system.
     */
    public Position getPositionAt(Point p) {
	if (p == null || mapUL == null || boardUL == null) {
	    System.out.println("getPositionAt: p = " + p + ", mapUL = " +
			       mapUL + ", boardUL = " + boardUL);
	}
        return new Position(mapUL.x + ((double) (p.x - boardUL.x)) / getZoomFactor(),
                mapUL.y + ((double) (p.y - boardUL.y)) / getZoomFactor());
    }


    /**
     *  Converts the given Position of the map to the actual screen Point in the
     *  board's coordinate system.
     *
     * @param  p  A position in the map coordinate system.
     * @return    The corresponding pixel in the board coordinate system.
     */
    public Point getScreenPoint(Position p) {
        return new Point(boardUL.x + (int) Math.floor((p.x - mapUL.x) * getZoomFactor()),
                boardUL.y + (int) Math.floor((p.y - mapUL.y) * getZoomFactor()));
    }


    /**
     *  Returns the bounding rectangle of the given piece. The rectangle is
     *  given in actual screen coordinates (i.e. the coordinates of the
     *  map-and-pieces component).
     *
     * @param  piece  A piece.
     * @return        A bounding rectangle for the piece.
     */
    public Rectangle getPieceBounds(Piece piece) {
        Image symbol = piece.getSymbol();
        if (symbol == null) {
            return new Rectangle(0, 0, 1, 1);
        }
        int width = symbol.getWidth(board);
        int height = symbol.getHeight(board);
        Point p = getScreenPoint(piece.getPosition());
        int x = p.x;
        int y = p.y;
        return new Rectangle(x - width / 2, y - height / 2, width, height);
    }


    /**
     *  Sets the map of the board.
     *
     * @param  m  Description of Parameter
     */
    public void loadMap(Map m) {
        map = m;
        mapUL = new Position(0.0, 0.0);
        boardUL = new Point(0, 0);
        zoom = 0;
        zoomInButton.setEnabled(true);
        zoomOutButton.setEnabled(true);
        adjustZoom(map.getCenter());
    }


    /**
     *  Description of the Method
     */
    public void clearMap() {
        if (map == null) {
            return;
        }
        map.clear();
        horizontalScrollbar.setEnabled(false);
        verticalScrollbar.setEnabled(false);
        zoomInButton.setEnabled(false);
        zoomOutButton.setEnabled(false);
    }


    /**
     *  Updates the given point of the map to the given color. Colors are given
     *  as integers as decribed in java.awt.Color.
     *
     * @param  x      Description of Parameter
     * @param  y      Description of Parameter
     * @param  color  Description of Parameter
     */
    public void mapUpdate(int x, int y, int color) {
        if (map == null) {
            return;
        }
        map.updatePoint(x, y, color);
    }


    /**
     *  Makes the given point of the map invisible. If it has already been set
     *  as invisible, nothing is done.
     *
     * @param  x  Description of Parameter
     * @param  y  Description of Parameter
     */
    public void hideMapPoint(int x, int y) {
        if (map == null) {
            return;
        }
        map.hidePoint(x, y);
    }


    /**
     *  Adds the given mouse listener the the area where the map and the pieces
     *  are painted. Use this method to assign the preferred behavior to the
     *  board (such as mouse clicks for selecting pieces).
     *
     * @param  ml  The new mouse listener.
     */
    public void addBoardMouseListener(MouseListener ml) {
        board.addMouseListener(ml);
    }


    /**
     *  Adds the given MouseMotionListener to the area of the board where the
     *  map and the pieces are painted.
     *
     * @param  mml  The new mouse motion listener.
     */
    public void addBoardMouseMotionListener(MouseMotionListener mml) {
        board.addMouseMotionListener(mml);
    }


    /**
     *  Repaints the part of the board where the map and pieces are.
     */
    public void repaintBoard() {
	// This is the straight-forward way things ought to be done:
        //board.repaint();

	// And this is what we have to do, since calling repaint() too
	// often slows Windows (but not UNIX) to a crawl:
	synchronized(this) {
	    repaint = true;
	}
    }


    /**
     *  Returns the size of the actual board, not counting the scroll bars and
     *  zoom buttons.
     *
     * @return    The size of the board.
     */
    public Dimension boardSize() {
        return board.getSize();
    }


    /**
     *  Computes the factor by which everything is zoomed in. 1.0 corresponds to
     *  100% (natural size), 3.0 to 300%, etc. 0.5 corresponds to 50%, which is
     *  actually zoomed out by a factor two.
     *
     * @return    The zoom factor.
     */
    private double getZoomFactor() {
        if (zoom >= 0) {
            return (double) (1 + zoom);
        } else {
            return 1.0 / ((double) (1 - zoom));
        }
    }


    /**
     *  Repaint the given area of the offscreen buffer.
     *
     * @param  d  The size of the area we want to repaint.
     */
    private void repaintOffscreenBuffer(Dimension d) {

        // Obtain a Graphics object
        Graphics g = offscreenBuffer.getGraphics();

        // If there is no map, just paint everything white
        if (map == null) {
            g.setColor(Color.white);
            g.fillRect(0, 0, d.width, d.height);
            return;
        }

        /*
         * Paint the map. This is a bit tricky, since we really need
         * to specify double coordinates in the map coordinate
         * system. The solution is to round in the right direction,
         * and paint somewhat more than we really need.
         */

        // The upper left and lower right corners in the automaton
        // coordinate system. We make sure to round _away_ from the
        // rectangle. This makes it somewhat bigger than what we
        // originally asked for.
        Dimension mapSize = map.getSize();
        Position ULCorner = getPositionAt(new Point(0, 0));
        Position LRCorner = getPositionAt(new Point(d.width - 1, d.height - 1));
        int sourceX1 = (int) Math.floor(Math.max(ULCorner.x, mapUL.x));
        int sourceY1 = (int) Math.floor(Math.max(ULCorner.y, mapUL.y));
        int sourceX2 = (int) Math.ceil(Math.min(LRCorner.x, (double) (mapSize.width - 1)));
        int sourceY2 = (int) Math.ceil(Math.min(LRCorner.y, (double) (mapSize.height - 1)));

        // The upper left and lower right corners in the pixel
        // coordinate system. To get exactly the rectangle we asked
        // for painted on screen, we make the destination rectangle
        // bigger than necessary, to exactly compensate for the
        // increased size of the source rectangle.
        Point dest1 = getScreenPoint(new Position((double) sourceX1, (double) sourceY1));
        Point dest2 = getScreenPoint(new Position((double) sourceX2, (double) sourceY2));
        int destX1 = dest1.x;
        int destY1 = dest1.y;
        int destX2 = dest2.x;
        int destY2 = dest2.y;

        // If the map doesn't cover the board completely, we need to
        // paint a black background.
        if (destX1 > 0 || destY1 > 0 || destX2 < (d.width - 1)
                 || destY2 < (d.height - 1)) {
            g.setColor(Color.black);
            g.fillRect(0, 0, d.width, d.height);
        }

        // Finally, draw the thing.
	if (map.getImage() != null) {
	    g.drawImage(map.getImage(), destX1, destY1, destX2, destY2,
			sourceX1, sourceY1, sourceX2, sourceY2, this);
	}

        // Draw lines between the queued actions of the selected piece
        g.setColor(Color.red);
        Enumeration positions = game.getSelectedPiecePositions();
        if (positions != null) {
            Point last = null;
            while (positions.hasMoreElements()) {
                Position pos = (Position) positions.nextElement();
                Point p = getScreenPoint(pos);
                if (last != null) {
                    g.drawLine(last.x, last.y, p.x, p.y);
                }
                last = p;
            }
        }

        // Paint all pieces within range
        Enumeration pieces =
                game.getPiecesIntersecting(new Rectangle(0, 0, d.width, d.height));
        while (pieces.hasMoreElements()) {
            Piece p = (Piece) pieces.nextElement();
            p.paint(g, getPieceBounds(p));
        }
    }


    /**
     *  Zooms in one step.
     */
    private void zoomIn() {
        Position center = game.getPreferredCenter();
        zoom++;
        adjustZoom(center);
    }


    /**
     *  Zooms out one step.
     */
    private void zoomOut() {
        Position center = game.getPreferredCenter();
        zoom--;
        adjustZoom(center);
    }


    /**
     *  Adjusts everything to a (possibly, but not necessarily) changed zoom
     *  value. This includes centering on the point returned by
     *  Game.getPreferredCenter().
     *
     * @param  center  Description of Parameter
     */
    private void adjustZoom(Position center) {

        double zf = getZoomFactor();

        // Get the center and size of the board and map. (The "center"
        // of the map isn't its actual center, but the "preferred"
        // center, which is either the position of the currently
        // selected piece or the position of the center of the board.)
        Position mapCenter = center;
        Dimension boardSize = boardSize();
        Point boardCenter = new Point(boardSize.width / 2, boardSize.height / 2);
        Dimension mapSize = map.getSize();

        // If we're sufficiently zoomed out that the entire map fits
        // on the board along an axis, the center should be the real
        // center, so that the map is centeded on the board.
        Position realMapCenter = map.getCenter();
        if (((double) mapSize.width) * zf <= ((double) boardSize.width)) {
            mapCenter.x = realMapCenter.x;
        }
        if (((double) mapSize.height) * zf <= ((double) boardSize.height)) {
            mapCenter.y = realMapCenter.y;
        }

        // Compute new upper left corner of visible portion of the
        // map.
        mapUL.x = Math.max(0.0, mapCenter.x - ((double) boardSize.width) / zf / 2.0);
        mapUL.y = Math.max(0.0, mapCenter.y - ((double) boardSize.height) / zf / 2.0);

        // Compute the point on the board where the upper left corner
        // of the map is.
        boardUL.x = (int) Math.max(0.0, boardCenter.x -
                ((double) mapSize.width) * zf / 2.0);
        boardUL.y = (int) Math.max(0.0, boardCenter.y -
                ((double) mapSize.height) * zf / 2.0);

        // Fix horizontal scrollbar.
        int minValue = 0;
        int maxValue = mapSize.width - 1;
        int tabLength = (int) Math.round(((double) boardSize.width) / zf);
        int tabPosition = (int) Math.round(mapUL.x);
        if (tabLength < maxValue) {
            horizontalScrollbar.removeAdjustmentListener(horScrollListener);
            horizontalScrollbar.setValues(tabPosition, tabLength, minValue, maxValue);
            horizontalScrollbar.setBlockIncrement(tabLength);
            horizontalScrollbar.addAdjustmentListener(horScrollListener);
            horizontalScrollbar.setEnabled(true);
        } else {
            horizontalScrollbar.setEnabled(false);
        }

        // Fix vertical scrollbar.
        minValue = 0;
        maxValue = mapSize.height - 1;
        tabLength = (int) Math.round(((double) boardSize.height) / zf);
        tabPosition = (int) Math.round(mapUL.y);
        if (tabLength < maxValue) {
            verticalScrollbar.removeAdjustmentListener(vertScrollListener);
            verticalScrollbar.setValues(tabPosition, tabLength, minValue, maxValue);
            verticalScrollbar.setBlockIncrement(tabLength);
            verticalScrollbar.addAdjustmentListener(vertScrollListener);
            verticalScrollbar.setEnabled(true);
        } else {
            verticalScrollbar.setEnabled(false);
        }
    }
}
