/***********************************************************************

GECCO, a platform for command and control gaming

Copyright (C) 2001 Henrik Brnhielm, Andreas Enblom, Niklas
Hallenfur, Karl Hasselstrm, Henrik Hgerstrm, Oskar Linde, Jing Fu
Zi and Jon slund

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
USA.

***********************************************************************/

package gecco.communication.messages;

import java.io.*;

/**
 *  This is the superclass to all messages.
 *
 * @author     Karl Hasselstrm
 * @created    July 25, 2001
 */
public abstract class Message {

    /**
     *  This method should write the message to the given
     *  <code>DataOutput</code>.
     *
     * @param out The output stream.
     */
    public abstract void write(DataOutput out) throws IOException;


    /**
     *  This factory method reads a message id from the input stream
     *  and calls the <code>read()</code> factory method of the
     *  corresponding subclass, which is responsible for reading the
     *  actual data.
     *
     * @param  in  The input stream.
     * @return An instance of <code>Message</code>.
     */
    public static Message read(DataInput in) throws IOException {

	// Determine the type of the message.
        int type = in.readInt();
	
	// Create a message object of that type.
	switch (type) {
	case MessageType.AbortAction:
	    return AbortActionMessage.read(in);
	case MessageType.AbortGame:
	    return AbortGameMessage.read(in);
	case MessageType.AcceptConnection:
	    return AcceptConnectionMessage.read(in);
	case MessageType.ActionBlocked:
	    return ActionBlockedMessage.read(in);
	case MessageType.ActionCompleted:
	    return ActionCompletedMessage.read(in);
	case MessageType.ActionFailed:
	    return ActionFailedMessage.read(in);
	case MessageType.ActionOnPosition:
	    return ActionOnPositionMessage.read(in);
	case MessageType.ActionOnUnit:
	    return ActionOnUnitMessage.read(in);
	case MessageType.ActionOnVoid:
	    return ActionOnVoidMessage.read(in);
	case MessageType.AvailableRoles:
	    return AvailableRolesMessage.read(in);
	case MessageType.Disconnect:
	    return DisconnectMessage.read(in);
	case MessageType.GetAvailableRoles:
	    return GetAvailableRolesMessage.read(in);
	case MessageType.GetMap:
	    return GetMapMessage.read(in);
	case MessageType.JoinAsRole:
	    return JoinAsRoleMessage.read(in);
	case MessageType.Map:
	    return MapMessage.read(in);
	case MessageType.MapPointInvisible:
	    return MapPointInvisibleMessage.read(in);
	case MessageType.MapPointsInvisible:
	    return MapPointsInvisibleMessage.read(in);
	case MessageType.MapUpdate:
	    return MapUpdateMessage.read(in);
	case MessageType.MapUpdates:
	    return MapUpdatesMessage.read(in);
	case MessageType.Ping:
	    return PingMessage.read(in);
	case MessageType.PingReply:
	    return PingReplyMessage.read(in);
	case MessageType.RejectConnection:
	    return RejectConnectionMessage.read(in);
	case MessageType.StartGame:
	    return StartGameMessage.read(in);
	case MessageType.TimeUpdate:
	    return TimeUpdateMessage.read(in);
	case MessageType.UnitDeleted:
	    return UnitDeletedMessage.read(in);
	case MessageType.UnitDestroyed:
	    return UnitDestroyedMessage.read(in);
	case MessageType.UnitMessage:
	    return UnitMessageMessage.read(in);
	case MessageType.UnitPositionUpdate:
	    return UnitPositionUpdateMessage.read(in);
	case MessageType.UnitPropertyUpdate:
	    return UnitPropertyUpdateMessage.read(in);
	case MessageType.UnitVisible:
	    return UnitVisibleMessage.read(in);
	default:
	    return new InvalidMessage();
 	}
    }
    
}
