/***********************************************************************

GECCO, a platform for command and control gaming

Copyright (C) 2001 Henrik Brnhielm, Andreas Enblom, Niklas
Hallenfur, Karl Hasselstrm, Henrik Hgerstrm, Oskar Linde, Jing Fu
Zi and Jon slund

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
USA.

***********************************************************************/

package gecco.server.actionprocessor;

import java.util.*;

import gecco.server.clock.*;
import gecco.server.core.*;
import gecco.server.unitmanager.*;
import gecco.server.visibility.*;
import gecco.util.*;

/**
 *  The action processor handles the scheduling and execution of actions for the
 *  units in the ongoing game.
 *
 * @author     Henrik Brnhielm
 * @author     Karl Hasselstrm
 * @author     Henrik Hgerstrm
 * @created    April 23, 2001
 */
public class ActionProcessor implements SignalRunnable {

    /**
     *  The priority queue used to hold the actions.
     */
    private TreeSet actionQueue;


    /**
     *  A (role, actionId) -> Action mapping.
     */
    private HashMap actionMap;


    /**
     *  Constructor for the ActionProcessor object
     *
     */
    public ActionProcessor() {
        actionQueue = new TreeSet();
	actionMap = new HashMap();
    }


    /**
     *  Returns the <code>TreeSet</code> object which is the queue that holds
     *  the current scheduled actions for the units in the game. This method is
     *  primarily used when another thread wants to synchronize on the queue
     *  object, preventing any other thread to add a new action to the queue.
     *
     * @return    The priority queue that is used for scheduling actions for
     *      units.
     */
    public TreeSet getActionQueue() {
        return actionQueue;
    }


    /**
     *  Creates a key for use in the actionMap.
     *
     * @return    An object that can be used as a key.
     */
    private static Object createKey(String role, int actionId) {
	Vector v = new Vector(2);
	v.add(role);
	v.add(new Integer(actionId));
        return v;
    }


    /**
     *  Creates a key for use in the actionMap.
     *
     * @return    An object that can be used as a key.
     */
    private static Object createKey(QueuedAction qA) {
        return createKey(qA.getFromRole(), qA.getAction().getActionHandle());
    }


    /**
     *  Gets the action with the given handle ordered by the given
     *  role. Will return null if there is no such action, or if it
     *  has already completed.
     *
     * @return The action.
     */
    public Action getAction(String role, int actionHandle) {
	QueuedAction qA = (QueuedAction) actionMap.get(createKey(role, actionHandle));
	if (qA == null) {
	    return null;
	}
	return qA.getAction();
    }


    /**
     *  Initiates an action for a unit.
     *
     * @param  fromRole  The commanding role which initiated the action.
     * @param  action    The Action class itself.
     */
    public void initiateAction(String fromRole, Action action) {
        QueuedAction qA = new QueuedAction(fromRole, action,
					   ReferenceHolder.getClock().gameTimeNow() + 0.01);
        synchronized (actionQueue) {
            actionQueue.add(qA);
	    //bl
	    //hr borde det insertas en qa och inte en a
	    actionMap.put(createKey(fromRole, action.getActionHandle()), qA);
            actionQueue.notifyAll();
        }
    }


    /**
     *  Executes an instantaneous action. An instantaneous action is an action
     *  that can be executed during the execution of another (scheduled) action.
     *  A unit can for example be executing a movement action, and fire a
     *  missile at another unit at the same time. The instantaneous action will
     *  never be queued. There is only one single call made to the <code>
     *  checkPoint()</code> method in the action class.
     *
     * @param  action  The class representing the instantaneous action.
     */
    public void executeInstantaneousAction(Action action) {
        // Synchronize on the queue so that nothing is happening with any units!
        synchronized (actionQueue) {
            // Get the Unit and the UnitType
            Unit u = action.getUnit();
            UnitType uT = u.getUnitType();

            // Save the coordinates for later use
            double oldx = u.getX();
            double oldy = u.getY();
            int oldIntX = (int) oldx;
            int oldIntY = (int) oldy;

            // Reset the update maps
            u.resetUpdatedActions();
            u.resetUpdatedProperties();

            // Do one iteration of the action!
            ActionReturn actRet = action.checkPoint();

            // Check for changes
            if (u.isDestroyed()) {
                // Unit is destroyed, act!!!
                ReferenceHolder.getUnitManager().deleteUnit(u, oldx, oldy);
                ReferenceHolder.getVisibilityFilter().deleteUnit(u, oldx, oldy);
            } else {
                // Check if the unit has moved, set new values for properties or
                // blocked one or more actions.
                checkChangedPosition(oldx, oldy, u);
                checkChangedProperties(u);
                checkActionBlocklist(u);
            }
            // Send the unit message, if the user returned one
            String unitMessage = actRet.getUnitMessage();
            if (unitMessage != null) {
                // Send to commander, observers and gods eye roles!
                ReferenceHolder.getVisibilityFilter().unitMessage(u, unitMessage);
            }
        }
    }


    /**
     *  This method handles an incoming <code>UnitEvent</code> coming from the
     *  automaton-layer, most probably due to a change of state in the automaton
     *  at position (x, y).
     *
     * @param  x          The x-coordinate of the automaton.
     * @param  y          The y-coordinate of the automaton.
     * @param  unitEvent  The UnitEvent class.
     */
    public void handleAutomatonUnitEvent(int x, int y, UnitEvent unitEvent) {
        // Synchronize on the queue so that nothing is happening with any units!
        synchronized (actionQueue) {
            // Gets an iterator over all units at discrete coordinate position.
            Iterator it = ReferenceHolder.getUnitManager().getUnitsAtAutomatonPosition(x, y).iterator();
            while (it.hasNext()) {
                Unit u = (Unit) it.next();
                UnitType uT = u.getUnitType();
                double oldx = u.getX();
                double oldy = u.getY();
                int oldIntX = (int) oldx;
                int oldIntY = (int) oldy;
                u.resetUpdatedActions();
                u.resetUpdatedProperties();
                uT.getEventHandler().handleEvent(unitEvent, u);
                // Check for changes...
                if (u.isDestroyed()) {
                    // Unit is destroyed, act!!!
		    ReferenceHolder.getUnitManager().deleteUnit(u, oldx, oldy);
		    ReferenceHolder.getVisibilityFilter().deleteUnit(u, oldx, oldy);
                } else {
                    checkChangedPosition(oldx, oldy, u);
                    checkChangedProperties(u);
                    checkActionBlocklist(u);
                }
            }
        }
    }


    /**
     *  This method handles an incoming UnitEvent which originates from another
     *  unit in the game.
     *
     * @param  unit       The affected unit.
     * @param  unitEvent  The UnitEvent class.
     */
    public void handleUnitToUnitEvent(Unit unit, UnitEvent unitEvent) {
        // Synch on the queue so that nothing is happening with any units!
        synchronized (actionQueue) {
            UnitType uT = unit.getUnitType();
            double oldx = unit.getX();
            double oldy = unit.getY();
            int oldIntX = (int) oldx;
            int oldIntY = (int) oldy;
            unit.resetUpdatedActions();
            unit.resetUpdatedProperties();
            uT.getEventHandler().handleEvent(unitEvent, unit);
            // Check for changes...
            if (unit.isDestroyed()) {
                // Unit is destroyed, act!!!
		ReferenceHolder.getUnitManager().deleteUnit(unit, oldx, oldy);
		ReferenceHolder.getVisibilityFilter().deleteUnit(unit, oldx, oldy);
            } else {
                checkChangedPosition(oldx, oldy, unit);
                checkChangedProperties(unit);
                checkActionBlocklist(unit);
            }
        }
    }


    /**
     *  Aborts an ongoing action for a unit.
     *
     * @param  fromRole      The commanding role
     * @param  actionHandle  The actionHandle
     * @return               <code>true</code> if the action was found and
     *      removed, <code>false</code> otherwise.
     */
    public boolean abortAction(String fromRole, int actionHandle) {
        synchronized (actionQueue) {
	    Object key = createKey(fromRole, actionHandle);
	    QueuedAction qA = (QueuedAction) actionMap.get(key);
	    if (qA == null) {
		return false;
	    } else {
		qA.getAction().getUnit().setActiveAction(null);
		actionQueue.remove(qA);
		actionMap.remove(key);
		return true;
	    }
	}
    }


    /**
     *  Main processing method for the execution of actions.
     */
    public void run() {
        while (true) {

	    // Check for signals.
	    switch (SignalThread.signalPending()) {
	    case SIGKILL:
	    case SIGQUIT:
		// Stop executing.
		return;
	    case SIGPAUSE:
		SignalThread.pauseHandler();
		SignalThread.clearSignal();
		break;
	    default:
		// Do nothing.
		SignalThread.clearSignal();
		break;
	    }

	    Clock clock = ReferenceHolder.getClock();
            QueuedAction qA = null;
            double nextExecutionTime = 0.0;

            // Synchronize on the queue.
            synchronized (actionQueue) {
                // Fetch current time!
                double curTime = clock.gameTimeNow();
                // See if there's anything in the queue!
                if (actionQueue.size() > 0) {
                    // Something is in the queue, see if it is due for execution!
                    qA = (QueuedAction) actionQueue.first();
                    if (qA.getTimeToExecute() <= curTime) {
                        // Yes indeed, remove the object from the queue!
                        actionQueue.remove(qA);
                    } else {
                        // Nope, it is not due yet, store the time for execution
                        // and leave it in the queue!
                        nextExecutionTime = qA.getTimeToExecute();
                        qA = null;
                    }
                }
            }

            // If qA isn't null, then we've found an action which is due for execution!
            if (qA != null) {
                // Execute a checkpoint for the action!
                Action act = qA.getAction();
                Unit u = ReferenceHolder.getUnitManager().getUnit(act.getUnitHandle());
                double oldx = u.getX();
                double oldy = u.getY();
                int oldIntX = (int) oldx;
                int oldIntY = (int) oldy;
                u.resetUpdatedActions();
                u.resetUpdatedProperties();

                ActionReturn actRet = act.checkPoint();
                // Check if things has changed...
                if (u.isDestroyed()) {
		    ReferenceHolder.getUnitManager().deleteUnit(u, oldx, oldy);
		    ReferenceHolder.getVisibilityFilter().deleteUnit(u, oldx, oldy);
                } else {
                    checkChangedPosition(oldx, oldy, u);
                    checkChangedProperties(u);
                    checkActionBlocklist(u);
                    checkActionReturn(qA, u, actRet);
                }
                // Yield to other threads before continuing.
                Thread.yield();
            } else {
                // Wait for notifications coming from the automaton queue,
                // or 'til it's time to execute the next scheduled action...
                double timeToSleep = nextExecutionTime - clock.gameTimeNow();
                synchronized (actionQueue) {
                    try {
                        if (actionQueue.size() == 0) {
			    actionQueue.wait();
                        } else {
			    clock.gameWait(actionQueue, timeToSleep);
			}
                    } catch (InterruptedException e) {
                        // We were woken early, perhaps someone sent us a signal?
                    }
                }
            }
        }
    }


    /**
     *  Adds an action to the action queue, with a given execution time.
     *
     * @param  qA               The action to be added.
     * @param  timeToExecution  The time until execution.
     */
    private void addUpdateToQueue(QueuedAction qA, double timeToExecution) {
        double timeToExecute = ReferenceHolder.getClock().gameTimeNow()
	    + timeToExecution;
        synchronized (actionQueue) {
            qA.setTimeToExecute(timeToExecute);
            actionQueue.add(qA);
	    actionMap.put(createKey(qA), qA);
        }
    }


    /**
     *  Check if a given unit has changed from a known last position. If the
     *  unit has moved, inform the unit manager and visibility manager of the
     *  update.
     *
     * @param  oldx  The last known x coordinate
     * @param  oldy  The last known x coordinate
     * @param  u     The unit.
     */
    private void checkChangedPosition(double oldx, double oldy, Unit u) {
        // Check if the unit has moved!
        if (oldx != u.getX() || oldy != u.getY()) {
            // Yes, it has moved!

            // Has it also moved when using a discrete-valued coordinate system?
            if ((int) oldx != u.getIntX() || (int) oldy != u.getIntY()) {
                // Inform the unitManager of the new position
                ReferenceHolder.getUnitManager().reportUpdatedUnitPosition(u, (int) oldx, (int) oldy);
            }

            // Tell the world.
            ReferenceHolder.getVisibilityFilter().updateUnitPosition(u,
								     oldx,
								     oldy,
								     u.getX(),
								     u.getY());
        }
    }


    /**
     *  Checks the ongoing actions of a given unit and notify the roles
     *  (clients) that commands the role about any updates.
     *
     * @param  u  The unit to check.
     */
    private void checkActionBlocklist(Unit u) {
        Set actions = u.getUpdatedActions();
	UnitType uType = u.getUnitType();
	Iterator it = actions.iterator();
	while (it.hasNext()) {
	    String thisAction = (String) it.next();
	    boolean isBlocked = u.getBlockedActions().contains(thisAction);
	    ActionDefinition actDef = uType.getActionDefinition(thisAction);
	    ReferenceHolder.getVisibilityFilter().actionBlocked(u, actDef, isBlocked);
	}
    }


    /**
     *  Checks all properties of the given unit, and notify all roles (clients)
     *  that observes the unit about any updates. Also notify God's Eye roles.
     *
     * @param  u  The unit to check.
     */
    private void checkChangedProperties(Unit u) {
        HashMap hM = u.getUpdatedProperties();
	Iterator itProps = hM.keySet().iterator();
	while (itProps.hasNext()) {
	    String thisKey = (String) itProps.next();
	    String thisValue = (String) hM.get(thisKey);
	    ReferenceHolder.getVisibilityFilter().unitPropertyChanged(u, thisKey, thisValue);
	}
    }


    /**
     *  Check if a given action, executed by a given unit, is completed, aborted
     *  or still ongoing, and take the necessary steps in each case.
     *
     * @param  qA      The action to check.
     * @param  u       The unit executing the action.
     * @param  actRet  The return from the last checkpoint call.
     */
    private void checkActionReturn(QueuedAction qA, Unit u, ActionReturn actRet) {
        String unitMessage = actRet.getUnitMessage();

        // Is the action completed?
        if (actRet instanceof ActionReturnCompleted) {
	    ReferenceHolder.getVisibilityFilter().unitMessage(u, unitMessage);
            ReferenceHolder.getVisibilityFilter().actionCompleted(qA.getAction(), true);
            u.setActiveAction(null);
	    actionMap.remove(createKey(qA));
        }

        // Is the action aborted?
        else if (actRet instanceof ActionReturnError) {
	    ReferenceHolder.getVisibilityFilter().unitMessage(u, unitMessage);
            ReferenceHolder.getVisibilityFilter().actionCompleted(qA.getAction(), false);
            u.setActiveAction(null);
	    actionMap.remove(createKey(qA));
        }

        // Is the action still ongoing?
        else if (actRet instanceof ActionReturnRequeue) {
            addUpdateToQueue(qA, ((ActionReturnRequeue) actRet).getTimeToWakeup());
	    ReferenceHolder.getVisibilityFilter().unitMessage(u, unitMessage);
        }
    }


    /**
     *  Outputs a message to the server log.
     *
     * @param  s  The message to output.
     */
    private static void log(String s) {
        ReferenceHolder.log.println("ActionProcessor\t: " + s);
    }


    /**
     *  Outputs a debug message to the server log.
     *
     * @param  s  The message to output.
     */
    private static void debug(String s) {
        ReferenceHolder.debug.println("ActionProcessor\t: " + s);
    }

}
