/***********************************************************************

GECCO, a platform for command and control gaming

Copyright (C) 2001 Henrik Brnhielm, Andreas Enblom, Niklas
Hallenfur, Karl Hasselstrm, Henrik Hgerstrm, Oskar Linde, Jing Fu
Zi and Jon slund

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
USA.

***********************************************************************/

package gecco.server.actionprocessor;

import gecco.server.core.*;

/**
 *  This class is a wrapper object that is stored in the priority queue in the
 *  ActionProcessor.
 *
 * @author     Karl Hasselstrm
 * @author     Henrik Hgerstrm
 * @created    April 23, 2001
 */
public class QueuedAction implements Comparable {
    /**
     *  The action.
     */
    private Action action;
    /**
     *  The role that ordered the action.
     */
    private String fromRole;
    /**
     *  The time when the action is supposed to be carried out, in
     *  game time. Or rather, the time when its next checkpoint is due
     *  (there may be several, but only one at a time is scheduled --
     *  the last thing a checkpoint does is schedule a new
     *  checkpoint).
     */
    private double timeToExecute;


    /**
     *  Creates a QueuedAction object.
     *
     * @param  fromRole              The commanding role.
     * @param  action                The action to be executed.
     * @param timeToExecute The time when the next checkpoint will be
     * executed.
     */
    public QueuedAction(String fromRole, Action action, double timeToExecute) {
        this.fromRole = fromRole;
        this.action = action;
        this.timeToExecute = timeToExecute;
    }


    /**
     *  Sets the TimeToExecute attribute of the QueuedAction object.
     *
     * @param  newTimeToExecute  The new TimeToExecute value.
     */
    public void setTimeToExecute(double newTimeToExecute) {
        timeToExecute = newTimeToExecute;
    }


    /**
     *  Gets the Action attribute of the QueuedAction object.
     *
     * @return    The Action value.
     */
    public Action getAction() {
        return action;
    }


    /**
     *  Gets the FromRole attribute of the QueuedAction object.
     *
     * @return    The FromRole value.
     */
    public String getFromRole() {
        return fromRole;
    }


    /**
     *  Gets the TimeToExecute attribute of the QueuedAction object.
     *
     * @return    The TimeToExecute value.
     */
    public double getTimeToExecute() {
        return timeToExecute;
    }


    /**
     *  Indicates whether some other object is "equal to" this one.
     *
     * @param  other  The other object.
     * @return <code>true</code> if this object is the same as the obj
     * argument; <code>false</code> otherwise.
     */
    public boolean equals(Object other) {
        return (this.compareTo(other) == 0);
    }


    /**
     *  Compares this object with the specified object for
     *  order. Returns a negative integer, zero, or a positive integer
     *  as this object is less than, equal to, or greater than the
     *  specified object.
     *
     * @param  other  The other object.
     * @return A negative integer, zero, or a positive integer as this
     * object is less than, equal to, or greater than the specified
     * object.
     */
    public int compareTo(Object a) {
        // Another object always comes before this one in a TreeSet
        if (!(a instanceof QueuedAction)) {
            return 1;
        }
        QueuedAction b = (QueuedAction) a;
        if (timeToExecute < b.getTimeToExecute()) {
	    return -1;
	} else if (timeToExecute > b.getTimeToExecute()) {
	    return 1;
	}
        return action.getActionHandle() - b.getAction().getActionHandle();
    }

}
