/***********************************************************************

GECCO, a platform for command and control gaming

Copyright (C) 2001 Henrik Brnhielm, Andreas Enblom, Niklas
Hallenfur, Karl Hasselstrm, Henrik Hgerstrm, Oskar Linde, Jing Fu
Zi and Jon slund

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
USA.

***********************************************************************/

package gecco.server.clock;

import gecco.server.clock.*;

/**
 *  Description of the Class
 *
 * @author     Karl Hasselstrm
 * @created    July 16, 2001
 */
public class TimeString {

    /**
     *  The clock that keeps track of time.
     */
    private Clock clock;
    /**
     *  Number of seconds per minute.
     */
    private final static long SECS_PER_MINUTE = 60;
    /**
     *  Number of seconds per hour.
     */
    private final static long SECS_PER_HOUR = 60 * 60;
    /**
     *  Number of seconds per day.
     */
    private final static long SECS_PER_DAY = 24 * 60 * 60;
    /**
     *  Number of seconds per year.
     */
    private final static long SECS_PER_YEAR = 365 * 24 * 60 * 60;


    /**
     *  Creates a new TimeString object which uses the given clock.
     *
     * @param  clock  The clock used to find out what time it is.
     */
    public TimeString(Clock clock) {
        this.clock = clock;
    }


    /**
     *  Returns a String specifying the current game time.
     *
     * @return    A String specifying the current game time.
     */
    public String getGameTime() {
        return getTimeString(clock.gameTimeNow(), clock.isFrozen());
    }


    /**
     *  Returns a String specifying the current real time.
     *
     * @return    A String specifying the current real time.
     */
    public String getRealTime() {
        return getTimeString(clock.realTimeNow(), false);
    }


    /**
     *  Returns the name of the basic unit of game time.
     *
     * @return    The basic unit of game time.
     */
    public static String getBasicTimeUnit() {
        return "s";
    }


    /**
     *  Returns a String specifying the given time.
     *
     * @param  seconds   The current time, in seconds since the beginning of
     *      time.
     * @param  isFrozen  Description of Parameter
     * @return           A String specifying the time.
     */
    public static String getTimeString(double seconds, boolean isFrozen) {
        long years = Math.round(Math.floor(seconds / SECS_PER_YEAR));
        seconds -= (double) years * SECS_PER_YEAR;

        long days = Math.round(Math.floor(seconds / SECS_PER_DAY));
        seconds -= (double) days * SECS_PER_DAY;

        long hours = Math.round(Math.floor(seconds / SECS_PER_HOUR));
        seconds -= (double) hours * SECS_PER_HOUR;

        long minutes = Math.round(Math.floor(seconds / SECS_PER_MINUTE));
        seconds -= (double) minutes * SECS_PER_MINUTE;

        return "" + years + "y" + days + "d" + hours + "h" +
                minutes + "m" + Math.round(seconds) + "s" +
                (isFrozen ? " (paused)" : "");
    }
}
