/***********************************************************************

GECCO, a platform for command and control gaming

Copyright (C) 2001 Henrik Brnhielm, Andreas Enblom, Niklas
Hallenfur, Karl Hasselstrm, Henrik Hgerstrm, Oskar Linde, Jing Fu
Zi and Jon slund

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
USA.

***********************************************************************/

package gecco.server.qmanager;

import gecco.server.automaton.*;
import gecco.server.clock.*;
import gecco.server.core.*;
import gecco.util.*;

import java.awt.Color;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.ResourceBundle;
import java.util.TreeSet;

/**
 *  This class manages the queue of AutomatonEvents and QueuedAutomatons.
 *
 * @author     Henrik Brnielm
 * @author     Karl Hasselstrm
 * @author     Henrik Hgerstrm
 * @created    April 23, 2001
 */
public class QManager implements SignalRunnable {

    /**
     *  Description of the Field
     */
    private TreeSet automatonQueue;
    /**
     *  Description of the Field
     */
    private HashMap timeToAutomatonMap;
    /**
     *  Description of the Field
     */
    private LinkedList eventQueue;


    /**
     *  Constructor for the QManager object
     *
     */
    public QManager() {
        automatonQueue = new TreeSet();
        eventQueue = new LinkedList();
        timeToAutomatonMap = new HashMap();
    }


    /**
     *  Gets the queue of automatons
     *
     * @return    The queue
     */
    public TreeSet getAutomatonQueue() {
        return automatonQueue;
    }


    /**
     *  Gets the queue of AutomatonEvents
     *
     * @return    The queue
     */
    public LinkedList getAutomatonEventQueue() {
        return eventQueue;
    }


    /**
     *  Adds an event to the event queue
     *
     * @param  event  The event to be added.
     * @param  x      The x coordinate of the position of the event.
     * @param  y      The y coordinate of the position of the event.
     */
    public void addEventToQueue(AutomatonEvent event, int x, int y) {
        if (!ReferenceHolder.getAutomatonMap().areValidCoordinates(x, y)) {
            throw new IllegalArgumentException("Invalid coordinates...");
        }
        QueuedEvent qE = new QueuedEvent(event, x, y);
        synchronized (eventQueue) {
            eventQueue.addLast(qE);
        }
        synchronized (automatonQueue) {
            automatonQueue.notifyAll();
        }
    }


    /**
     *  Main processing method for the QManager object
     */
    public void run() {

        /*
         * for (int x = 0; x < aMap.getXBound(); x++) {
         * for (int y = 0; y < aMap.getYBound(); y++) {
         * addUpdateToQueue(x, y, 100);
         * }
         * }
         */

        while (true) {

	    Clock clock = ReferenceHolder.getClock();

	    // Check for signals.
	    switch (SignalThread.signalPending()) {
	    case SIGKILL:
	    case SIGQUIT:
		// Stop executing.
		return;
	    case SIGPAUSE:
		SignalThread.pauseHandler();
		SignalThread.clearSignal();
		break;
	    default:
		// Do nothing.
		SignalThread.clearSignal();
		break;
	    }

            QueuedEvent qE = null;
            QueuedAutomaton qA = null;
            double nextExecutionTime = 0.0;
            synchronized (eventQueue) {
                if (eventQueue.size() > 0) {
                    qE = (QueuedEvent) eventQueue.removeFirst();
                }
            }
            if (qE != null) {
                // Process event...
                int x = qE.getX();
                int y = qE.getY();
                Automaton aut = ReferenceHolder.getAutomatonMap().getAutomaton(x, y);
                int oldState = aut.getState();
                AutomatonReturn autRet = aut.handleEvent(qE.getAutomatonEvent());
                if (aut.hasChanged()) {
                    // Notify
                    Color c = aut.getCurrentColor();
                    ReferenceHolder.getVisibilityFilter().updateAutomatonState(x, y,
									       c.getRGB());
                }
                checkAutomatonReturn(autRet, x, y);
            } else {
		// There was no element in the event queue, check if
		// it's time to update an automaton
                synchronized (automatonQueue) {
                    double curTime = clock.gameTimeNow();
                    if (automatonQueue.size() > 0) {
                        qA = (QueuedAutomaton) automatonQueue.first();
                        // Remove key in HashMap...
                        synchronized (timeToAutomatonMap) {
                            Integer mapKey = new Integer(qA.getX() *
							 ReferenceHolder.getAutomatonMap().getHeight()
							 + qA.getY());
                            timeToAutomatonMap.remove(mapKey);
                        }
                        if (qA.getTimeToExecute() <= curTime) {
                            automatonQueue.remove(qA);
                        } else {
                            nextExecutionTime = qA.getTimeToExecute();
                            qA = null;
                        }
                    }
                }
                if (qA != null) {
                    Automaton aut = ReferenceHolder.getAutomatonMap().getAutomaton(qA.getX(), qA.getY());
                    int oldState = aut.getState();
                    AutomatonReturn autRet =
			aut.update(ReferenceHolder.getAutomatonMap().getNeighbourStates(qA.getX(),
											qA.getY()));
                    if (aut.hasChanged()) {
                        Color c = aut.getCurrentColor();
                        ReferenceHolder.getVisibilityFilter().updateAutomatonState(qA.getX(),
										   qA.getY(),
										   c.getRGB());
                    }
                    checkAutomatonReturn(autRet, qA.getX(), qA.getY());
                }
            }

            // Yield to other threads if we've "done something"...
            if (qE != null || qA != null) {
                Thread.yield();
            } else {
                // Wait for notifications coming from the automaton queue,
                // or 'til it's time to execute next scheduled automaton update...
                double timeToSleep = nextExecutionTime - clock.gameTimeNow();
                synchronized (automatonQueue) {
                    try {
                        if (automatonQueue.size() == 0) {
                            automatonQueue.wait();
                        } else {
			    clock.gameWait(automatonQueue, timeToSleep);
			}
                    } catch (InterruptedException e) {
                        // Someone woke us; a signal perhaps?
                    }
                }
            }
        }
    }


    /**
     *  Adds an automaton to the automaton queue.
     *
     * @param  x                The x coordinate of the automaton.
     * @param  y                The y coordinate of the automaton.
     * @param  timeToExecution  The time until the automaton should be updated.
     */
    private void addUpdateToQueue(int x, int y, double timeToExecution) {
	Clock clock = ReferenceHolder.getClock();
        Integer mapKey = new Integer(x * ReferenceHolder.getAutomatonMap().getHeight() + y);
        double timeToExecute = clock.gameTimeNow() + timeToExecution;
        synchronized (automatonQueue) {
            QueuedAutomaton qA = (QueuedAutomaton) timeToAutomatonMap.get(mapKey);
            if (qA != null) {
                if (qA.getTimeToExecute() >= timeToExecute) {
                    automatonQueue.remove(qA);
                    qA.setTimeToExecute(timeToExecute);
                    automatonQueue.add(qA);
                }
            } else {
                qA = new QueuedAutomaton(x, y, timeToExecute);
                automatonQueue.add(qA);
                timeToAutomatonMap.put(mapKey, qA);
            }
            automatonQueue.notifyAll();
        }
    }


    /**
     *  Checks the return from an automaton update method, and possibly inserts
     *  the automaton and its neighbours into the queue.
     *
     * @param  autRet  The return object from the update method.
     * @param  x       The x coordinate of the automaton that was updated.
     * @param  y       The y coordinate of the automaton that was updated.
     */
    private void checkAutomatonReturn(AutomatonReturn autRet, int x, int y) {
	AutomatonMap aMap = ReferenceHolder.getAutomatonMap();
        UnitEvent event = autRet.getUnitEvent();
        if (event != null) {
            ReferenceHolder.getActionProcessor().handleAutomatonUnitEvent(x, y, event);
        }
        if (autRet instanceof AutomatonReturnNoAction) {
            // Nothing...
            return;
        }
        if (autRet instanceof AutomatonReturnQueueSelective) {
            double[][] times = ((AutomatonReturnQueueSelective) autRet).getTimeToExecutions();
            for (int ix = 0; ix < 3; ix++) {
                for (int iy = 0; iy < 3; iy++) {
                    if (times[ix][iy] > 0.0) {
                        // Queue this automaton
                        int xpos = x + ix - 1;
                        int ypos = y + iy - 1;
                        addUpdateToQueue(x, y, times[ix][iy]);
                    }
                }
            }
            return;
        }
        if (autRet instanceof AutomatonReturnQueueSelf ||
                autRet instanceof AutomatonReturnQueueSelfAndNeighbours) {
            addUpdateToQueue(x, y, autRet.getSelfTimeToExecution());
        }

        if (autRet instanceof AutomatonReturnQueueNeighbours ||
                autRet instanceof AutomatonReturnQueueSelfAndNeighbours) {
            int xFrom = (x - 1) >= 0 ? x - 1 : 0;
            int yFrom = (y - 1) >= 0 ? y - 1 : 0;
            int xTo = (x + 1) < aMap.getWidth() ? x + 1 : aMap.getWidth() - 1;
            int yTo = (y + 1) < aMap.getHeight() ? y + 1 : aMap.getHeight() - 1;
            for (int curX = xFrom; curX <= xTo; curX++) {
                for (int curY = yFrom; curY <= yTo; curY++) {
                    if (curX != x || curY != y) {
                        addUpdateToQueue(curX, curY, autRet.getNeighboursTimeToExecution());
                    }
                }
            }
        }
    }

}
