/***********************************************************************

GECCO, a platform for command and control gaming

Copyright (C) 2001 Henrik Brnhielm, Andreas Enblom, Niklas
Hallenfur, Karl Hasselstrm, Henrik Hgerstrm, Oskar Linde, Jing Fu
Zi and Jon slund

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
USA.

***********************************************************************/

package gecco.server.startup;

import java.io.*;
import java.text.*;

import gecco.server.core.*;
import gecco.server.clock.*;
import gecco.server.startup.*;

/**
 *  A rudimentary command prompt for the server.
 *
 * @author     Karl Hasselstrm
 * @created    July 13, 2001
 */
public class CommandPrompt implements Runnable {
    /**
     *  The command prompt loop.
     */
    public void run() {

	// Show the current server status.
	printStatus();

        // Display prompts and execute commands until the quit flag is
        // set.
        boolean quit = false;
        while (!quit) {
	    try {
		String command = prompt();
		if (command.equals("description")) {
		    printDescription();
		} else if (command.equals("help")) {
		    printHelp();
		} else if (command.equals("name")) {
		    printName();
		} else if (command.equals("quit")) {
		    quit = true;
		} else if (command.startsWith("speed")) {
		    adjustSpeed(command.substring(5).trim());
		} else if (command.equals("start")) {
		    ReferenceHolder.getClock().freeze(false);
		} else if (command.equals("status")) {
		    printStatus();
		} else if (command.equals("stop")) {
		    ReferenceHolder.getClock().freeze(true);
		} else {
		    ReferenceHolder.out.print("Unknown command. Type \"help\" " +
					      "for a list of available commands.");
		}
	    } catch (IOException e) {
		ReferenceHolder.out.println("An exception occurred:");
		ReferenceHolder.out.println(e);
                quit = true;
	    }
        }
	
        // Shut down the server.
        ReferenceHolder.out.println();
        ReferenceHolder.out.println("Shutting down server . . . ");
        StartServer.shutdown();
        ReferenceHolder.out.println("done.");
        ReferenceHolder.out.println("Shutting down the Java virtual machine.");
	System.exit(0);
    }


    /**
     *  This method displays a prompt and asks the user to input one
     *  line of text.
     *
     * @return A line of text, with leading and trailing whitespace
     * trimmed.
     */
    private String prompt() throws IOException {
        ReferenceHolder.out.println();
        ReferenceHolder.out.println();
        ReferenceHolder.out.print("> ");
        return ReferenceHolder.in.readLine().trim();
    }


    private void adjustSpeed(String newSpeed) {
	Clock clock = ReferenceHolder.getClock();

	// Normal speed is a special case.
	if (newSpeed.equals("normal")) {
	    clock.setNormalSpeed();
	    return;
	}

	// Try to interpret the argumant as a number.
	try {
	    double speed = Double.parseDouble(newSpeed);
	    clock.setSpeed(speed*clock.getNormalSpeed());
	} catch (NumberFormatException e) {
	    ReferenceHolder.out.println("Error. The speed must either be \"normal\" or a number.");
	}

	// Done.
	return;
    }


    private void printDescription() {
        ReferenceHolder.out.println(ReferenceHolder.getGameName());
        ReferenceHolder.out.println();
        ReferenceHolder.out.println(ReferenceHolder.getGameDescription());
    }
    

    /**
     *  This method displays help for the prompt.
     */
    private void printHelp() {
        ReferenceHolder.out.println("Available commands:");
        ReferenceHolder.out.println("description  Prints a short description of the current game.");
        ReferenceHolder.out.println("help         Show this help text.");
        ReferenceHolder.out.println("name         Prints the name of the current game.");
        ReferenceHolder.out.println("quit         Shut down the server.");
        ReferenceHolder.out.println("speed        Adjusts the speed of the simulation.");
        ReferenceHolder.out.println("start        Starts the game time if it was previously stopped.");
        ReferenceHolder.out.println("status       Shows current status of the server.");
        ReferenceHolder.out.println("stop         Stops the game time.");
    }


    private void printName() {
        ReferenceHolder.out.println(ReferenceHolder.getGameName());
    }


    private void printStatus() {
	ReferenceHolder.out.println("Current real time is " +
				    ReferenceHolder.getTimeString().getRealTime());
	ReferenceHolder.out.println("Current game time is " +
				    ReferenceHolder.getTimeString().getGameTime());
	if (ReferenceHolder.getClock().isFrozen()) {
	    ReferenceHolder.out.println("Game time is stopped.");
	} else {
	    double speed = ReferenceHolder.getClock().getSpeed();
	    double normalSpeed = ReferenceHolder.getClock().getNormalSpeed();
	    DecimalFormat format = new DecimalFormat("0.00");
	    ReferenceHolder.out.println("Game time is running at " + format.format(speed) + " " +
					ReferenceHolder.getTimeString().getBasicTimeUnit() +
					" per second of real time.");
	    if (speed >= normalSpeed*1.05) {
		ReferenceHolder.out.println("(" + format.format(speed/normalSpeed) +
					    " times faster than normal.)");
	    } else if (speed <= normalSpeed/1.05) {
		ReferenceHolder.out.println("(" + format.format(normalSpeed/speed) +
					    " times slower than normal.)");
	    }
	}
    }

}
