/***********************************************************************

GECCO, a platform for command and control gaming

Copyright (C) 2001 Henrik Brnhielm, Andreas Enblom, Niklas
Hallenfur, Karl Hasselstrm, Henrik Hgerstrm, Oskar Linde, Jing Fu
Zi and Jon slund

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
USA.

***********************************************************************/

package gecco.server.visibility;

import gecco.server.core.*;

import java.awt.Color;

/**
 *  This interface acts as a filter between the server and the outside
 *  world. The server calls the methods here whenever something
 *  happens, and the class implementing this interface is then
 *  responsible for censoring this information before passing it on to
 *  the clients.
 *
 * @author     Karl Hasselstrm
 * @created July 10, 2001
 */
public interface VisibilityFilter {

    /**
     *  This method is to be called whenever an automaton changes
     *  state.
     *
     * @param  x  The X coordinate of the automaton.
     * @param  y  The Y coordinate of the automaton.
     * @param c The new color of the automaton.
     */
    public void updateAutomatonState(int x, int y, int c);


    /**
     *  This method is to be called whenever a unit changes its
     *  position.
     *
     * @param  unit  The unit.
     * @param  oldX  The unit's old X coordinate.
     * @param  oldY  The unit's old Y coordinate.
     * @param  newX  The unit's new X coordinate.
     * @param  newY  The unit's new Y coordinate.
     */
    public void updateUnitPosition(Unit unit, double oldX, double oldY,
            double newX, double newY);


    /**
     *  This method is to be called whenever a unit is permanently
     *  taken off the map (typically, when it is killed).
     *
     * @param  unit        The unit.
     * @param  lastKnownX  The unit's last X coordinate.
     * @param  lastKnownY  The unit's last Y coordinate.
     */
    public void deleteUnit(Unit unit, double lastKnownX, double lastKnownY);


    /**
     *  This method should be called whenever an action becomes
     *  blocked or unblocked.
     *
     * @param  unit       The unit whose action gets blocked or unblocked.
     * @param  action     The action that is now blocked or unblocked.
     * @param isBlocked true if the action is blocked, false if it is unblocked.
     */
    public void actionBlocked(Unit unit, ActionDefinition action, boolean isBlocked);


    /**
     *  This method is to be called whenever the execution of an
     *  action is finished.
     *
     * @param  action         The action.
     * @param wasSuccessful   true if the action was successful, false
     *                             if it failed.
     */
    public void actionCompleted(Action action, boolean wasSuccessful);


    /**
     *  This method is to be called when a role requests that an
     *  action be carried out, and that is for some reason impossible.
     *
     * @param  role           The role.
     * @param  actionHandle   The action handle.
     */
    public void actionRejected(String role, int actionHandle);


    /**
     *  This method may be called to pass some information to the
     *  owner of a unit. If message is null, no message is sent.
     *
     * @param  unit     The unit.
     * @param  message  The message to be delivered.
     */
    public void unitMessage(Unit unit, String message);


    /**
     *  This method may be called to pass some information to a
     *  specific role regarding a unit. If message is null, no message
     *  is sent.
     *
     * @param  role     The role.
     * @param  unit     The unit.
     * @param message The message to be delivered.
     */
    public void roleMessage(String role, Unit unit, String message);


    /**
     *  This method may be called to tell the clients that some time
     *  has passed.
     *
     * @param time  A String representing the current game time.
     */
    public void timeUpdate(String time);


    /**
     *  This method should be called whenever a unit property changes.
     *
     * @param  unit      The unit whose property changed.
     * @param  key       The property that has changed.
     * @param  newValue  The new value of the property.
     */
    public void unitPropertyChanged(Unit unit, String key, String newValue);


    /**
     *  This method causes the server to resend all data to a role
     *  that the clients playing that role should know (and should be
     *  caching). This is primarily used to provide clients with
     *  initial data when they join the game, but it could also be
     *  requested by a client who has forgotten things.
     *
     *  The data is sent to all clients playing that role.
     *
     * @param  unit      The unit whose property changed.
     * @param  key       The property that has changed.
     * @param newValue The new value of the property.
     */
    public void resendAllData(String role);

}
